@@ -7,7 +7,7 @@ import torch.nn.functional as F
 from mushroom_rl.algorithms.agent import Agent
 from mushroom_rl.approximators import Regressor
 from mushroom_rl.approximators.parametric import TorchApproximator
-from mushroom_rl.utils.torch import to_float_tensor
+from mushroom_rl.utils.torch import to_float_tensor, update_optimizer_parameters
 from mushroom_rl.utils.minibatches import minibatch_generator
 from mushroom_rl.utils.dataset import parse_dataset, compute_J
 from mushroom_rl.utils.value_functions import compute_gae
@@ -132,3 +132,7 @@ class PPO(Agent):
                 avg_rwd, logging_verr, logging_ent, logging_kl))
             tqdm.write(
                 '--------------------------------------------------------------------------------------------------')
+
+    def _post_load(self):
+        if self._optimizer is not None:
+            update_optimizer_parameters(self._optimizer, list(self.policy.parameters()))
