@@ -23,7 +23,8 @@ class SACPolicy(Policy):
     the internals calculations of the SAC algorithm.
 
     """
-    def __init__(self, mu_approximator, sigma_approximator, min_a, max_a):
+    def __init__(self, mu_approximator, sigma_approximator, min_a, max_a,
+                 log_std_min, log_std_max):
         """
         Constructor.
 
@@ -36,6 +37,8 @@ class SACPolicy(Policy):
                 for each component;
             max_a (np.ndarray): a vector specifying the maximum action value
                 for each component.
+            log_std_min (float): Min value for the policy log std;
+            log_std_max (float): Max value for the policy log std;
 
         """
         self._mu_approximator = mu_approximator
@@ -44,6 +47,11 @@ class SACPolicy(Policy):
         self._delta_a = to_float_tensor(.5 * (max_a - min_a), self.use_cuda)
         self._central_a = to_float_tensor(.5 * (max_a + min_a), self.use_cuda)
 
+        self._log_std_min = log_std_min
+        self._log_std_max = log_std_max
+
+        self._eps_log_prob = 1e-6
+
         use_cuda = self._mu_approximator.model.use_cuda
 
         if use_cuda:
@@ -54,7 +62,8 @@ class SACPolicy(Policy):
             _mu_approximator='mushroom',
             _sigma_approximator='mushroom',
             _delta_a='torch',
-            _central_a='torch'
+            _central_a='torch',
+            _eps_log_prob='primitive'
         )
 
     def __call__(self, state, action):
@@ -101,7 +110,7 @@ class SACPolicy(Policy):
 
         if compute_log_prob:
             log_prob = dist.log_prob(a_raw).sum(dim=1)
-            log_prob -= torch.log(1. - a.pow(2) + 1e-6).sum(dim=1)
+            log_prob -= torch.log(1. - a.pow(2) + self._eps_log_prob).sum(dim=1)
             return a_true, log_prob
         else:
             return a_true
@@ -120,6 +129,8 @@ class SACPolicy(Policy):
         """
         mu = self._mu_approximator.predict(state, output_tensor=True)
         log_sigma = self._sigma_approximator.predict(state, output_tensor=True)
+        # Bound the log_std
+        log_sigma = torch.clamp(log_sigma, self._log_std_min, self._log_std_max)
         return torch.distributions.Normal(mu, log_sigma.exp())
 
     def entropy(self, state=None):
@@ -174,6 +185,18 @@ class SACPolicy(Policy):
         """
         return self._mu_approximator.model.use_cuda
 
+    def parameters(self):
+        """
+        Returns the trainable policy parameters, as expected by torch
+        optimizers.
+
+        Returns:
+            List of parameters to be optimized.
+
+        """
+        return chain(self._mu_approximator.model.network.parameters(),
+                     self._sigma_approximator.model.network.parameters())
+
 
 class SAC(DeepAC):
     """
@@ -185,7 +208,9 @@ class SAC(DeepAC):
     def __init__(self, mdp_info, actor_mu_params, actor_sigma_params,
                  actor_optimizer, critic_params, batch_size,
                  initial_replay_size, max_replay_size, warmup_transitions, tau,
-                 lr_alpha, target_entropy=None, critic_fit_params=None):
+                 lr_alpha,
+                 log_std_min=-20, log_std_max=2,
+                 target_entropy=None, critic_fit_params=None):
         """
         Constructor.
 
@@ -207,6 +232,8 @@ class SAC(DeepAC):
                 replay memory to start the policy fitting;
             tau (float): value of coefficient for soft updates;
             lr_alpha (float): Learning rate for the entropy coefficient;
+            log_std_min (float): Min value for the policy log std;
+            log_std_max (float): Max value for the policy log std;
             target_entropy (float, None): target entropy for the policy, if
                 None a default value is computed ;
             critic_fit_params (dict, None): parameters of the fitting algorithm
@@ -245,7 +272,9 @@ class SAC(DeepAC):
         policy = SACPolicy(actor_mu_approximator,
                            actor_sigma_approximator,
                            mdp_info.action_space.low,
-                           mdp_info.action_space.high)
+                           mdp_info.action_space.high,
+                           log_std_min,
+                           log_std_max)
 
         self._init_target(self._critic_approximator,
                           self._target_critic_approximator)
@@ -336,12 +365,7 @@ class SAC(DeepAC):
         return q
 
     def _post_load(self):
-        if self._optimizer is not None:
-            self._parameters = list(
-                chain(self.policy._mu_approximator.model.network.parameters(),
-                      self.policy._sigma_approximator.model.network.parameters()
-                )
-            )
+        self._update_optimizer_parameters(self.policy.parameters())
 
     @property
     def _alpha(self):
