@@ -4,22 +4,15 @@ import numpy as np
 
 from mushroom_rl.algorithms.agent import Agent
 from mushroom_rl.approximators.parametric.torch_approximator import *
-from mushroom_rl.approximators.regressor import Ensemble, Regressor
+from mushroom_rl.approximators.regressor import Regressor
 from mushroom_rl.utils.replay_memory import PrioritizedReplayMemory, ReplayMemory
 
 
-class DQN(Agent):
-    """
-    Deep Q-Network algorithm.
-    "Human-Level Control Through Deep Reinforcement Learning".
-    Mnih V. et al.. 2015.
-
-    """
+class AbstractDQN(Agent):
     def __init__(self, mdp_info, policy, approximator, approximator_params,
                  batch_size, target_update_frequency,
                  replay_memory=None, initial_replay_size=500,
-                 max_replay_size=5000, fit_params=None, n_approximators=1,
-                 clip_reward=True):
+                 max_replay_size=5000, fit_params=None, clip_reward=True):
         """
         Constructor.
 
@@ -40,15 +33,12 @@ class DQN(Agent):
                 memory;
             fit_params (dict, None): parameters of the fitting algorithm of the
                 approximator;
-            n_approximators (int, 1): the number of approximator to use in
-                ``AveragedDQN``;
             clip_reward (bool, True): whether to clip the reward or not.
 
         """
         self._fit_params = dict() if fit_params is None else fit_params
 
         self._batch_size = batch_size
-        self._n_approximators = n_approximators
         self._clip_reward = clip_reward
         self._target_update_frequency = target_update_frequency
 
@@ -67,19 +57,10 @@ class DQN(Agent):
 
         apprx_params_train = deepcopy(approximator_params)
         apprx_params_target = deepcopy(approximator_params)
-        self.approximator = Regressor(approximator, **apprx_params_train)
-        self.target_approximator = Regressor(approximator,
-                                             n_models=self._n_approximators,
-                                             **apprx_params_target)
-        policy.set_q(self.approximator)
 
-        if self._n_approximators == 1:
-            self.target_approximator.set_weights(
-                self.approximator.get_weights())
-        else:
-            for i in range(self._n_approximators):
-                self.target_approximator[i].set_weights(
-                    self.approximator.get_weights())
+        self._initialize_regressors(approximator, apprx_params_train,
+                                    apprx_params_target)
+        policy.set_q(self.approximator)
 
         self._add_save_attr(
             _fit_params='pickle',
@@ -102,7 +83,7 @@ class DQN(Agent):
         if self._n_updates % self._target_update_frequency == 0:
             self._update_target()
 
-    def _fit_standard(self, dataset):
+    def _fit_standard(self, dataset, approximator=None):
         self._replay_memory.add(dataset)
         if self._replay_memory.initialized:
             state, action, reward, next_state, absorbing, _ = \
@@ -114,9 +95,12 @@ class DQN(Agent):
             q_next = self._next_q(next_state, absorbing)
             q = reward + self.mdp_info.gamma * q_next
 
-            self.approximator.fit(state, action, q, **self._fit_params)
+            if approximator is None:
+                self.approximator.fit(state, action, q, **self._fit_params)
+            else:
+                approximator.fit(state, action, q, **self._fit_params)
 
-    def _fit_prioritized(self, dataset):
+    def _fit_prioritized(self, dataset, approximator=None):
         self._replay_memory.add(
             dataset, np.ones(len(dataset)) * self._replay_memory.max_priority)
         if self._replay_memory.initialized:
@@ -132,16 +116,31 @@ class DQN(Agent):
 
             self._replay_memory.update(td_error, idxs)
 
-            self.approximator.fit(state, action, q, weights=is_weight,
-                                  **self._fit_params)
+            if approximator is None:
+                self.approximator.fit(state, action, q, weights=is_weight,
+                                      **self._fit_params)
+            else:
+                approximator.fit(state, action, q, weights=is_weight,
+                                 **self._fit_params)
+
+    def draw_action(self, state):
+        action = super().draw_action(np.array(state))
+
+        return action
+
+    def _initialize_regressors(self, approximator, apprx_params_train,
+                               apprx_params_target):
+        self.approximator = Regressor(approximator, **apprx_params_train)
+        self.target_approximator = Regressor(approximator,
+                                             **apprx_params_target)
+        self._update_target()
 
     def _update_target(self):
         """
         Update the target network.
 
         """
-        self.target_approximator.set_weights(
-            self.approximator.get_weights())
+        self.target_approximator.set_weights(self.approximator.get_weights())
 
     def _next_q(self, next_state, absorbing):
         """
@@ -155,16 +154,7 @@ class DQN(Agent):
             Maximum action-value for each state in ``next_state``.
 
         """
-        q = self.target_approximator.predict(next_state)
-        if np.any(absorbing):
-            q *= 1 - absorbing.reshape(-1, 1)
-
-        return np.max(q, axis=1)
-
-    def draw_action(self, state):
-        action = super(DQN, self).draw_action(np.array(state))
-
-        return action
+        raise NotImplementedError
 
     def _post_load(self):
         if isinstance(self._replay_memory, PrioritizedReplayMemory):
@@ -175,54 +165,15 @@ class DQN(Agent):
         self.policy.set_q(self.approximator)
 
 
-class DoubleDQN(DQN):
+class DQN(AbstractDQN):
     """
-    Double DQN algorithm.
-    "Deep Reinforcement Learning with Double Q-Learning".
-    Hasselt H. V. et al.. 2016.
+    Deep Q-Network algorithm.
+    "Human-Level Control Through Deep Reinforcement Learning".
+    Mnih V. et al.. 2015.
 
     """
     def _next_q(self, next_state, absorbing):
-        q = self.approximator.predict(next_state)
-        max_a = np.argmax(q, axis=1)
-
-        double_q = self.target_approximator.predict(next_state, max_a)
-        if np.any(absorbing):
-            double_q *= 1 - absorbing
-
-        return double_q
-
-
-class AveragedDQN(DQN):
-    """
-    Averaged-DQN algorithm.
-    "Averaged-DQN: Variance Reduction and Stabilization for Deep Reinforcement
-    Learning". Anschel O. et al.. 2017.
-
-    """
-    def __init__(self, mdp_info, policy, approximator, **params):
-        super().__init__(mdp_info, policy, approximator, **params)
-
-        self._n_fitted_target_models = 1
-
-        self._add_save_attr(_n_fitted_target_models='primitive')
-
-        assert len(self.target_approximator) > 1
-
-    def _update_target(self):
-        idx = self._n_updates // self._target_update_frequency\
-              % self._n_approximators
-        self.target_approximator[idx].set_weights(
-            self.approximator.get_weights())
-
-        if self._n_fitted_target_models < self._n_approximators:
-            self._n_fitted_target_models += 1
-
-    def _next_q(self, next_state, absorbing):
-        q = list()
-        for idx in range(self._n_fitted_target_models):
-            q.append(self.target_approximator.predict(next_state, idx=idx))
-        q = np.mean(q, axis=0)
+        q = self.target_approximator.predict(next_state)
         if np.any(absorbing):
             q *= 1 - absorbing.reshape(-1, 1)
 
