@@ -21,7 +21,7 @@ class Gym(Environment):
     are managed in a separate class.
 
     """
-    def __init__(self, name, horizon, gamma, **env_args):
+    def __init__(self, name, horizon, gamma, wrappers=None, **env_args):
         """
         Constructor.
 
@@ -29,6 +29,7 @@ class Gym(Environment):
              name (str): gym id of the environment;
              horizon (int): the horizon;
              gamma (float): the discount factor;
+             wrappers (list): list of wrappers to apply over the environment;
              **env_args: other gym environment parameters.
 
         """
@@ -40,6 +41,9 @@ class Gym(Environment):
             self._close_at_stop = False
 
         self.env = gym.make(name, **env_args)
+        if wrappers is not None:
+            for wrapper in wrappers:
+                self.env = wrapper(self.env)
 
         self.env._max_episode_steps = np.inf  # Hack to ignore gym time limit.
 
